==> Net-BitTorrent-0.041.tar.gz: Net-BitTorrent-0.041/scripts/bittorrent.pl <==
#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;
use Pod::Usage;
use Carp qw[croak carp];
use Time::HiRes qw[sleep];
use lib q[../lib];
use Net::BitTorrent::Protocol qw[:types];
use Net::BitTorrent;
$|++;
my ($VERSION, $dir, $chk, $int, $help, $port, $ver, @tor, %opts)
    = (sprintf(q[%.3f], (qw$Rev: 40 $)[1] / 1000), q[./], 1, 1, 0);
GetOptions(q[help|?]       => \$help,
           q[torrent|t=s@] => \@tor,
           q[port:i]       => \$port,
           q[directory:s]  => \$dir,
           q[check!]       => \$chk,
           q[version]      => \$ver,
           q[options=s%]   => sub { $opts{$_[1]} = $_[2]; }    # private
) or pod2usage(2);
@tor = grep {-f} @tor, @ARGV;
$ver && exit printf <<VER, $0, $VERSION, $Net::BitTorrent::VERSION, $^V, $^O;
%s version %.3f
Net::BitTorrent version %s
Perl version %vd on %s
VER
pod2usage({-verbose => 99,
           -sections =>
               q[NAME|Description|Synopsis|Options|Author|License and Legal],
          }
    )
    if $help
        or !@tor;
my $bt = new Net::BitTorrent({LocalPort => $port})
    or croak sprintf q[Failed to create Net::BitTorrent object (%s)], $^E;
$bt->on_event(
    q[incoming_packet],
    sub {
        trans_status(q[REJECTED], $_[1]) if $_[1]->{q[Type]} eq REJECT;
        trans_status(q[RECEIVED], $_[1]) if $_[1]->{q[Type]} eq PIECE;
    }
);
$bt->on_event(
    q[outgoing_packet],
    sub {
        trans_status(q[REQUESTING], $_[1]) if $_[1]->{q[Type]} eq REQUEST;
        trans_status(q[CANCELING ], $_[1]) if $_[1]->{q[Type]} eq CANCEL;
        trans_status(q[SENDING   ], $_[1]) if $_[1]->{q[Type]} eq PIECE;
    }
);
$bt->on_event(q[piece_hash_pass], sub { piece_status(q[pass], $_[1]); });
$bt->on_event(q[piece_hash_fail], sub { piece_status(q[fail], $_[1]); });
for my $path (@tor) {
    stat print qq[Loading '$path'...];
    my $obj = $bt->add_torrent({Path => $path, BaseDir => $dir})
        || carp(qq[Cannot load '$path': $^E]) && next;
    print qq[Hash checking...\n] and $obj->hashcheck if $chk;
    printf qq[Loaded '$path' [%s...%s%s]\n],
        ($obj->infohash =~ (m[^(.{4}).+(.{4})$])),
        ($obj->private ? q[|No DHT] : q[]);
    for (keys %opts) { $obj->$_($opts{$_}) if $obj->can($_); }
}
$SIG{q[INT]} = sub {
    $int = ($int + 1 > time) ? exit : time;
    print qq[\n--> Press Ctrl-C again within 3 seconds to exit <--\n]
        . $bt->_as_string(1)
        . (join q[\n], map { $_->_as_string(1) } values %{$bt->torrents});
};
for (keys %opts) { $bt->$_($opts{$_}) if $bt->can($_); }
$bt->do_one_loop(0.25) && sleep(0.50) while 1;

sub piece_status {
    my ($msg, $args) = @_;
    my $torrent = $args->{q[Torrent]};
    return printf qq[%s: %04d|%s|%4d/%4d|% 3.2f%%\r],
        $msg, $args->{q[Index]}, $torrent->_as_string(),
        (scalar grep {$_} split q[], unpack(q[b*], $torrent->bitfield)),
        $torrent->piece_count,
        (((  (scalar grep {$_} split q[], unpack q[b*], $torrent->bitfield)
           / ($torrent->piece_count)
          )
         ) * 100
        );
}

sub trans_status {
    printf(qq[%-10s p:%15s:%-5d i:%4d o:%7d l:%5d  \r],
           shift,
           $_[0]->{q[Peer]}->_host,
           $_[0]->{q[Peer]}->_port,
           $_[0]->{q[Payload]}->{q[Index]},
           $_[0]->{q[Payload]}->{q[Offset]},
           $_[0]->{q[Payload]}->{q[Length]}
    );
}

=pod

=head1 NAME

bittorrent - Very basic example BitTorrent client

=head1 Description

This is a B<very> basic demonstration of a full C<Net::BitTorrent>-based
client.

=head1 Synopsis

 bittorrent file.torrent
   or
 bittorrent [options] [file ...]

 Options:
   -t     --torrent     .torrent file to load
   -p     --port        TCP/UDP port opened for incoming connections
   -d     --directory   Base directory to store downloaded files
   --no-check           Skip integrity check at start
   --options            Advanced settings
   -?     --help        Display full documentation
   --version            Display version information

=head1 Options

=over 8

=item B<--torrent>

Open this .torrent file.

You may pass several -torrent parameters and load more than one .torrent
torrent.

=item B<--port>

Port number opened to the world for incoming connections.  This defaults
to C<0> and lets L<Net::BitTorrent|Net::BitTorrent> bind to a random,
unused port.

=item B<--directory>

Relative or absolute directory used as a base directory for storage.  By
default, this is the current working directory.

Please see L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> for
related information.

=item B<--no-check>

If found, the files will not be checked for integrity and we assume that
we have none of the data of this torrent.

=item B<--options>

Allows otherwise private settings to be changed.  For example, to set the
upload bandwidth limit...

  bittorrent --options _set_max_ul_rate=8192 [...]

You may pass several C<--options> parameters.

=item B<--help>

Print the manual page and exit.

=item B<--version>

Guess.

=back

=head1 Author

Sanko Robinson <sanko@cpan.org> - http://sankorobinson.com/

CPAN ID: SANKO

=head1 License and Legal

Copyright (C) 2008 by Sanko Robinson E<lt>sanko@cpan.orgE<gt>

This program is free software; you can redistribute it and/or modify
it under the terms of The Artistic License 2.0.  See the F<LICENSE>
file included with this distribution or
http://www.perlfoundation.org/artistic_license_2_0.  For
clarification, see http://www.perlfoundation.org/artistic_2_0_notes.

When separated from the distribution, all POD documentation is covered
by the Creative Commons Attribution-Share Alike 3.0 License.  See
http://creativecommons.org/licenses/by-sa/3.0/us/legalcode.  For
clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.

Neither this module nor the L<Author|/Author> is affiliated with
BitTorrent, Inc.

=for svn $Id: bittorrent.pl 40 2008-12-02 04:25:26Z sanko@cpan.org $

=cut

==> Net-BitTorrent-0.042.tar.gz: Net-BitTorrent-0.042/scripts/bittorrent.pl <==
#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;
use Pod::Usage;
use Carp qw[croak carp];
use Time::HiRes qw[sleep];
use lib q[../lib];
use Net::BitTorrent::Protocol qw[:types];
use Net::BitTorrent;
$|++;
my ($VERSION, $dir, $chk, $int, $help, $port, $ver, @tor, %opts)
    = (sprintf(q[%.3f], (qw$Rev: 42 $)[1] / 1000), q[./], 1, 1, 0);
GetOptions(q[help|?]       => \$help,
           q[torrent|t=s@] => \@tor,
           q[port:i]       => \$port,
           q[directory:s]  => \$dir,
           q[check!]       => \$chk,
           q[version]      => \$ver,
           q[options=s%]   => sub { $opts{$_[1]} = $_[2]; }    # private
) or pod2usage(2);
@tor = grep {-f} @tor, @ARGV;
$ver && exit printf <<VER, $0, $VERSION, $Net::BitTorrent::VERSION, $^V, $^O;
%s version %.3f
Net::BitTorrent version %s
Perl version %vd on %s
VER
pod2usage({-verbose  => 99,
           -sections => q[Synopsis|Author],
          }
    )
    if $help
        or !@tor;
my $bt = new Net::BitTorrent({LocalPort => $port})
    or croak sprintf q[Failed to create Net::BitTorrent object (%s)], $^E;
$bt->on_event(
    q[incoming_packet],
    sub {
        trans_status(q[REJECTED], $_[1]) if $_[1]->{q[Type]} eq REJECT;
        trans_status(q[RECEIVED], $_[1]) if $_[1]->{q[Type]} eq PIECE;
    }
);
$bt->on_event(
    q[outgoing_packet],
    sub {
        trans_status(q[REQUESTING], $_[1]) if $_[1]->{q[Type]} eq REQUEST;
        trans_status(q[CANCELING ], $_[1]) if $_[1]->{q[Type]} eq CANCEL;
        trans_status(q[SENDING   ], $_[1]) if $_[1]->{q[Type]} eq PIECE;
    }
);
$bt->on_event(q[piece_hash_pass], sub { piece_status(q[pass], $_[1]); });
$bt->on_event(q[piece_hash_fail], sub { piece_status(q[fail], $_[1]); });
for my $path (@tor) {
    stat print qq[Loading '$path'...];
    my $obj = $bt->add_torrent({Path => $path, BaseDir => $dir})
        || carp(qq[Cannot load '$path': $^E]) && next;
    print qq[Hash checking...\n] and $obj->hashcheck if $chk;
    printf qq[Loaded '$path' [%s...%s%s]\n],
        ($obj->infohash =~ (m[^(.{4}).+(.{4})$])),
        ($obj->private ? q[|No DHT] : q[]);
    for (keys %opts) { $obj->$_($opts{$_}) if $obj->can($_); }
}
$bt->on_event(q[file_error], sub { warn q[ERROR: ] . $_[1]->{q[Message]} });
$SIG{q[INT]} = sub {
    $int = ($int + 1 > time) ? exit : time;
    print qq[\n--> Press Ctrl-C again within 3 seconds to exit <--\n]
        . $bt->as_string(1)
        . (join q[\n], map { $_->as_string(1) } values %{$bt->torrents});
};
for (keys %opts) { $bt->$_($opts{$_}) if $bt->can($_); }
$bt->do_one_loop(0.25) && sleep(0.50) while 1;

sub piece_status {
    my ($msg, $args) = @_;
    my $torrent = $args->{q[Torrent]};
    return printf qq[%s: %04d|%s|%4d/%4d|% 3.2f%%\r],
        $msg, $args->{q[Index]}, $torrent->as_string(),
        (scalar grep {$_} split q[], unpack(q[b*], $torrent->bitfield)),
        $torrent->piece_count,
        (((  (scalar grep {$_} split q[], unpack q[b*], $torrent->bitfield)
           / ($torrent->piece_count)
          )
         ) * 100
        );
}

sub trans_status {
    printf(qq[%-10s p:%15s:%-5d i:%4d o:%7d l:%5d  \r],
           shift,
           $_[0]->{q[Peer]}->_host,
           $_[0]->{q[Peer]}->_port,
           $_[0]->{q[Payload]}->{q[Index]},
           $_[0]->{q[Payload]}->{q[Offset]},
           $_[0]->{q[Payload]}->{q[Length]}
    );
}

=pod

=head1 NAME

bittorrent - Very basic example BitTorrent client

=head1 Description

This is a B<very> basic demonstration of a full C<Net::BitTorrent>-based
client.

=head1 Synopsis

 bittorrent file.torrent
   or
 bittorrent [options] [file ...]

 Options:
   -t     --torrent     .torrent file to load
   -p     --port        TCP/UDP port opened for incoming connections
   -d     --directory   Base directory to store downloaded files
   --no-check           Skip integrity check at start
   --options            Advanced settings
   -?     --help        Display full documentation
   --version            Display version information

To get client-wide progress updates, press C<Ctrl+C>.  For more, see
perldoc.

=head1 Options

=over 8

=item B<--torrent>

Open this .torrent file.

You may pass several -torrent parameters and load more than one .torrent
torrent.

=item B<--port>

Port number opened to the world for incoming connections.  This defaults
to C<0> and lets L<Net::BitTorrent|Net::BitTorrent> bind to a random,
unused port.

=item B<--directory>

Relative or absolute directory used as a base directory for storage.  By
default, this is the current working directory.

Please see L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> for
related information.

=item B<--no-check>

If found, the files will not be checked for integrity and we assume that
we have none of the data of this torrent.

=item B<--options>

Allows otherwise private settings to be changed.  For example, to set the
upload bandwidth limit...

  bittorrent --options _set_max_ul_rate=8192 [...]

You may pass several C<--options> parameters.

=item B<--help>

Print the manual page and exit.

=item B<--version>

Guess.

=back

=head1 Author

Sanko Robinson <sanko@cpan.org> - http://sankorobinson.com/

CPAN ID: SANKO

=head1 License and Legal

Copyright (C) 2008 by Sanko Robinson E<lt>sanko@cpan.orgE<gt>

This program is free software; you can redistribute it and/or modify
it under the terms of The Artistic License 2.0.  See the F<LICENSE>
file included with this distribution or
http://www.perlfoundation.org/artistic_license_2_0.  For
clarification, see http://www.perlfoundation.org/artistic_2_0_notes.

When separated from the distribution, all POD documentation is covered
by the Creative Commons Attribution-Share Alike 3.0 License.  See
http://creativecommons.org/licenses/by-sa/3.0/us/legalcode.  For
clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.

Neither this module nor the L<Author|/Author> is affiliated with
BitTorrent, Inc.

=for svn $Id: bittorrent.pl 42 2008-12-05 04:54:43Z sanko@cpan.org $

=cut
